/*
 * Copyright (C) 2003 Jean-Philippe Hergott <hergott@free.fr>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <errno.h>

#include "callbacks.h"
#include "interface.h"
#include "support.h"

// variable local
GList *list_tmprackRow;
GList *list_tmprackCol;

void
on_xcave_window_button_config_clicked                      (GtkButton       *button,
                                        gpointer         user_data)
{

  FILE *f;
  gchar filename[256];

    XCAVE_DEBUG_B;
	/* 
	 * desactive window_xcave
	 */
	gtk_widget_set_sensitive(window_xcave, FALSE);

	window_config = create_window_config();
	
        signal_id_NbRow = gtk_signal_connect (GTK_OBJECT (lookup_widget (window_config, "ConfNbRow")),
		                              "changed",
                                              GTK_SIGNAL_FUNC (on_ConfNbRow_value_changed), 
					      NULL);
        signal_id_NbCol = gtk_signal_connect (GTK_OBJECT (lookup_widget (window_config, "ConfNbCol")),
		                              "changed",
                                              GTK_SIGNAL_FUNC (on_ConfNbCol_value_changed), 
					      NULL);
        signal_id_NbRack = gtk_signal_connect (GTK_OBJECT (lookup_widget (window_config, "ConfNbRack")),
		                              "changed",
                                              GTK_SIGNAL_FUNC (on_ConfNbRack_value_changed), 
					      NULL);
        signal_id_NumRack = gtk_signal_connect (GTK_OBJECT (lookup_widget (window_config, "confNumRack")),
		                              "changed",
                                              GTK_SIGNAL_FUNC (on_confNumRack_value_changed), 
					      NULL);
	memset(filename,'\0',sizeof(filename));
	sprintf(filename,"%s/.xcaverc",getenv("HOME"));
	if ((f = fopen (filename, "r")) == NULL) {
		if ((f= fopen ("/etc/xcave.conf", "r")) == NULL) {
			quick_message (_(" Configuration file error "));
		}
	}
	// configuration de numrack en fonction de nbrack
	gtk_spin_button_set_range           (GTK_SPIN_BUTTON (lookup_widget (window_config,  "confNumRack")),
                                                         1.0,
                                                         (gint) rackNb);
	gtk_entry_set_text (GTK_ENTRY (lookup_widget (window_config, "PathStock")), (gchar *) DBPATH);
	gtk_entry_set_text (GTK_ENTRY (lookup_widget (window_config, "ConfVers")), (gchar *) VERSION);
	gtk_entry_set_text (GTK_ENTRY (lookup_widget (window_config, "ConfLengthentry")), (gchar *) HEIGHT);
	gtk_entry_set_text (GTK_ENTRY (lookup_widget (window_config, "ConfWidthentry")), (gchar *) WIDTH);

	gtk_signal_handler_block (GTK_OBJECT (lookup_widget (window_config, "ConfNbRow")), signal_id_NbRow);
	gtk_signal_handler_block (GTK_OBJECT (lookup_widget (window_config, "ConfNbCol")), signal_id_NbCol);
	gtk_signal_handler_block (GTK_OBJECT (lookup_widget (window_config, "ConfNbRack")), signal_id_NbRack);
	gtk_signal_handler_block (GTK_OBJECT (lookup_widget (window_config, "confNumRack")), signal_id_NumRack);

	gtk_spin_button_set_value (GTK_SPIN_BUTTON (lookup_widget (window_config, "ConfQt")), (gint) QT);
	gtk_spin_button_set_value (GTK_SPIN_BUTTON (lookup_widget (window_config, "ConfNbRack")), (gint) rackNb);
	gtk_spin_button_set_value (GTK_SPIN_BUTTON (lookup_widget (window_config, "ConfNbRow")), (gint) g_list_nth_data(list_rackRow,0));
	gtk_spin_button_set_value (GTK_SPIN_BUTTON (lookup_widget (window_config, "ConfNbCol")), (gint) g_list_nth_data(list_rackCol,0));

	gtk_signal_handler_unblock (GTK_OBJECT (lookup_widget (window_config, "ConfNbRow")), signal_id_NbRow);
	gtk_signal_handler_unblock (GTK_OBJECT (lookup_widget (window_config, "ConfNbCol")), signal_id_NbCol);
	gtk_signal_handler_unblock (GTK_OBJECT (lookup_widget (window_config, "ConfNbRack")), signal_id_NbRack);
	gtk_signal_handler_unblock (GTK_OBJECT (lookup_widget (window_config, "confNumRack")), signal_id_NumRack);

	// copie de la liste colonne et row dans une liste temporaire
	list_tmprackRow = g_list_copy(list_rackRow);
	list_tmprackCol = g_list_copy(list_rackCol);
	gtk_widget_show (window_config);
	
    XCAVE_DEBUG_E;
}


void
on_window_config_button_ok_clicked                      (GtkButton       *button,
                                        gpointer         user_data)
{
  GtkSpinButton *spinqt;

    XCAVE_DEBUG_B;
  // recopie de la liste temporaire dans la liste
  list_rackRow = g_list_copy(list_tmprackRow);
  list_rackCol = g_list_copy(list_tmprackCol);
  
  strcpy (DBPATH, gtk_entry_get_text (GTK_ENTRY (lookup_widget (window_config, "PathStock"))));
  strcpy (HEIGHT, gtk_entry_get_text (GTK_ENTRY (lookup_widget (window_config, "ConfLengthentry"))));
  strcpy (WIDTH,  gtk_entry_get_text (GTK_ENTRY (lookup_widget (window_config, "ConfWidthentry"))));
  spinqt          = GTK_SPIN_BUTTON (lookup_widget (window_config, "ConfNbRack"));
  rackNb        = (gint) gtk_spin_button_get_value_as_int (spinqt);

  /*
   * Mise a jour et affichage de l'arbre (country/region/appellation) si le parametre a change
   */
  if (QT != (gint) gtk_spin_button_get_value_as_int (GTK_SPIN_BUTTON (lookup_widget (window_config, "ConfQt")))) {
	/*
	 * need to update storeview and storelist
	 */
	GtkWidget		*treelist;
	GtkWidget		*treeview;
	GtkListStore	*storelist;
	GtkTreeStore	*storeview;

        QT = (gint) gtk_spin_button_get_value_as_int (GTK_SPIN_BUTTON (lookup_widget (window_config, "ConfQt")));

	treelist = lookup_widget(window_xcave,"TreeList");
	storelist	= (GtkListStore *) gtk_tree_view_get_model((GtkTreeView *)treelist);
	gtk_list_store_clear(storelist);

	treeview = lookup_widget(window_xcave,"TreeView");
	storeview	=(GtkTreeStore *)  gtk_tree_view_get_model((GtkTreeView *)treeview);
	gtk_tree_store_clear(storeview);
    
	xcave_fill_model (storeview, storelist);

  }
  gtk_object_destroy (GTK_OBJECT (window_config));

  /* 
   * reactive window_xcave
   */
  gtk_widget_set_sensitive(window_xcave, TRUE);

  /*
   * Resizing de la fenetre pour prise en compte des nouvelles valeurs
   */
  gtk_window_set_default_size (GTK_WINDOW (window_xcave), atoi(WIDTH), atoi(HEIGHT));

    XCAVE_DEBUG_E;
}


void
on_window_config_button_cancel_clicked                  (GtkButton       *button,
                                        gpointer         user_data)
{

  gtk_object_destroy (GTK_OBJECT (window_config));
  
   /* 
    * reactive window_xcave
    */
   gtk_widget_set_sensitive(window_xcave, TRUE);
}

void
on_window_config_destroy               (GtkObject       *object,
                                        gpointer         user_data)
{
  gtk_object_destroy (GTK_OBJECT (window_config));
  g_list_free(list_tmprackRow);
  g_list_free(list_tmprackCol);	
  /* 
   * reactive window_xcave
   */
  gtk_widget_set_sensitive(window_xcave, TRUE);

}

void
on_ConfNbRow_value_changed(GtkSpinButton *spinbutton,gpointer       user_data){
    XCAVE_DEBUG_B;
   gtk_signal_handler_block (GTK_OBJECT (lookup_widget (window_config, "ConfNbRow")), signal_id_NbRow);
   gint nbRow      = (gint) gtk_spin_button_get_value_as_int (spinbutton);
   gint numRack    = (gint) gtk_spin_button_get_value_as_int( GTK_SPIN_BUTTON (lookup_widget (window_config,"confNumRack")));
   list_tmprackRow = g_list_delete_link (list_tmprackRow, g_list_nth (list_tmprackRow, numRack-1));
   list_tmprackRow = g_list_insert(list_tmprackRow, (gpointer)nbRow,numRack-1);
   gtk_signal_handler_unblock (GTK_OBJECT (lookup_widget (window_config, "ConfNbRow")), signal_id_NbRow);
    XCAVE_DEBUG_E;
}

void
on_ConfNbCol_value_changed(GtkSpinButton *spinbutton,gpointer       user_data){
    XCAVE_DEBUG_B;
   gtk_signal_handler_block (GTK_OBJECT (lookup_widget (window_config, "ConfNbCol")), signal_id_NbCol);
   gint nbCol      = (gint) gtk_spin_button_get_value_as_int (spinbutton);
   gint numRack    = (gint) gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON (lookup_widget (window_config,"confNumRack")));
   list_tmprackCol = g_list_delete_link (list_tmprackCol, g_list_nth (list_tmprackCol, numRack-1));
   list_tmprackCol = g_list_insert(list_tmprackCol, (gpointer)nbCol,numRack-1);
   gtk_signal_handler_unblock (GTK_OBJECT (lookup_widget (window_config, "ConfNbCol")), signal_id_NbCol);
    XCAVE_DEBUG_E;

}
void
on_ConfNbRack_value_changed(GtkSpinButton *spinbutton,gpointer       user_data){
    XCAVE_DEBUG_B;
  gtk_signal_handler_block (GTK_OBJECT (lookup_widget (window_config, "ConfNbRack")), signal_id_NbRack);

  /*
   * le nombre de casier peut etre modifier soit par le clavier (Text) soit par la souris (SpinButton)
   * il convient donc de recuperer la valeur text au prealable a toute autre operation
   */
  gint tmprackNb = atoi((const gchar*) gtk_entry_get_text(GTK_ENTRY(spinbutton)));	

  gtk_spin_button_set_range(GTK_SPIN_BUTTON (lookup_widget (window_config,"confNumRack")), 1.0, (gint) tmprackNb);

  while (g_list_length(list_tmprackRow) < tmprackNb){
      /*
      * par defaut ajout d'un casier de 10*5
      */
      gpointer i = g_malloc(sizeof(gint));
      i=(gchar *)10;
      gpointer j = g_malloc(sizeof(gint));
      j=(gchar *)5;
      list_tmprackRow = g_list_append(list_tmprackRow, (gpointer)i);
      list_tmprackCol = g_list_append(list_tmprackCol, (gpointer)j);
  }

  /*
   * En cas de modification Text il y a deux passages dans cette fonction 
   *   le 1er tmprackNb = 0 (mise à blanc de la zone)
   *   le 2sd valeur saisie
   * donc lors du 1er passage il faut faire attention a ne pas vider les listes Row et Col
   */
  while ((g_list_length(list_tmprackRow) > tmprackNb) && (tmprackNb != 0)) {
      list_tmprackRow = g_list_delete_link (list_tmprackRow, g_list_nth(list_tmprackRow,g_list_length(list_tmprackRow) - 1));
      list_tmprackCol = g_list_delete_link (list_tmprackCol, g_list_nth(list_tmprackCol,g_list_length(list_tmprackCol) - 1));
  }


  gtk_signal_handler_unblock (GTK_OBJECT (lookup_widget (window_config, "ConfNbRack")), signal_id_NbRack);
    XCAVE_DEBUG_E;
}

void
on_confNumRack_value_changed(GtkSpinButton *spinbutton,gpointer       user_data){
    XCAVE_DEBUG_B;
  gtk_signal_handler_block (GTK_OBJECT (lookup_widget (window_config, "confNumRack")), signal_id_NumRack);
  gint numRack =  (gint) gtk_spin_button_get_value_as_int (spinbutton);
  gtk_spin_button_set_value(GTK_SPIN_BUTTON (lookup_widget (window_config, "ConfNbRow")),(gint) g_list_nth_data(list_tmprackRow,numRack-1));
  gtk_spin_button_set_value(GTK_SPIN_BUTTON (lookup_widget (window_config,"ConfNbCol")),(gint) g_list_nth_data(list_tmprackCol,numRack-1));
  gtk_signal_handler_unblock (GTK_OBJECT (lookup_widget (window_config, "confNumRack")), signal_id_NumRack);
    XCAVE_DEBUG_E;
}
