/*
 * Copyright (C) 2003 Jean-Philippe Hergott <hergott@free.fr>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <errno.h>

#include <gtk/gtk.h>

#include "callbacks.h"
#include "interface.h"
#include "support.h"

GList *tmp_list_combo_region;
GList *tmp_list_combo_country;
GList *tmp_list_combo_app;

/***************************************************************************
 * Armements des signaux "changed" sur les champs country,region,appellation
 * GtkWindows: 'window_detail'
 * Comment: 
 */
void
xcave_signal_connect()
{
    XCAVE_DEBUG_B;
  signal_id_app = gtk_signal_connect (GTK_OBJECT (lookup_widget (window_detail, "vinapentry")),
		                              "changed",
                                              GTK_SIGNAL_FUNC (on_vinapentry_changed), 
					      NULL);
  signal_id_reg = gtk_signal_connect (GTK_OBJECT (lookup_widget (window_detail, "vinregentry")),
		                              "changed",
                                              GTK_SIGNAL_FUNC (on_vinregentry_changed), 
					      NULL);
  signal_id_pay = gtk_signal_connect (GTK_OBJECT (lookup_widget (window_detail, "wineCountryEntry")),
		                              "changed",
                                              GTK_SIGNAL_FUNC (on_wineCountryEntry_changed), 
					      NULL);
    XCAVE_DEBUG_E;
}

/***************************************************************************
 * Traitement d'un signal
 * GtkWindows: 'xcave'
 * Comment: Gestion des changements de taille de la fenetre
 */
void
on_xcave_check_resize                  (GtkContainer    *container,
                                        gpointer         user_data)
{
   gint height = 0;
   gint width = 0;

   gdk_window_get_size( container->widget.window , &width, &height);

   sprintf(WIDTH,"%d",width);
   sprintf(HEIGHT,"%d",height);
   
   /* 
    * Should be with gtk-2.0
    * gtk_window_get_size( GTK_WINDOW (container->widget) , &width, &height);
    */
}


void
on_window_detail_check_resize                 (GtkContainer    *container,
                                        gpointer         user_data)
{

   gint height = 0;
   gint width = 0;

   gdk_window_get_size( container->widget.window , &width, &height);

   sprintf(WindowsDetailWidth,"%d",width);
   sprintf(WindowsDetailHeight,"%d",height);
   
}

void
on_window_rack_check_resize                 (GtkContainer    *container,
                                        gpointer         user_data)
{

   gint height = 0;
   gint width = 0;

   gdk_window_get_size( container->widget.window , &width, &height);

   sprintf(WindowsRackWidth,"%d",width);
   sprintf(WindowsRackHeight,"%d",height);
   
}


/* *****************************************************************************
 * Traitement des combo vinaplist et vinreglist suite a une modification ou une 
 * saisie de wineCountryList
 */
void
xcave_find_C_app(gpointer	list_app_data, gpointer regitems)
{
	gchar *ptr_appitems;

	if (strcasecmp(regitems, list_app_data) == 0) {	
		/* need to fillup combo country for this region */
		ptr_appitems = g_malloc(appellation_size);
		strncpy(ptr_appitems, list_app_data + region_size, appellation_size);
		tmp_list_combo_app = g_list_append(tmp_list_combo_app,ptr_appitems);
	}
}

void
xcave_find_C_region(gpointer	list_region_data, gpointer countryItems)
{
	char *ptr_regitems;

	if (strcasecmp(countryItems, list_region_data) == 0) {
		
		/* need to fillup combo country for this region */
		ptr_regitems = g_malloc(region_size);
		strncpy(ptr_regitems, list_region_data + country_size, region_size);
		tmp_list_combo_region = g_list_append(tmp_list_combo_region,ptr_regitems);

		/* need to know originated country */
		g_list_foreach(list_region_appellation,xcave_find_C_app,ptr_regitems);
	}

}

void
xcave_find_C_country(gpointer	list_country_data, gpointer countryItems)
{
	if (strcasecmp(countryItems, list_country_data) == 0) {
		/* need to fillup combo country for this region */
		g_list_foreach(list_country_region,xcave_find_C_region,countryItems);
	}
}

/***************************************************************************
 * Traitement d'un signal
 * Windows: 'detail'
 * GtkEntry: wineCountryEntry
 * Comment: Selection d'un country different dans la GtkCombo wineCountryList
 */
void
on_wineCountryEntry_changed                (GtkEditable     *editable,
                                        gpointer         user_data)
{
	gchar	*ptr_items;
	gchar	appitems[appellation_size];
	gchar	regitems[region_size];
	gchar	countryItems[country_size];

    XCAVE_DEBUG_B;

	/* 
	* Un pays est selectionne - 
	*   Si region pas encore selectionne - alors on selectionne les regions du pays
	*   Si app pas encore selectionne - alors on selectionne les app associe au region
	*/

	memset(countryItems,'\0',country_size);
	strcpy (countryItems,	gtk_entry_get_text (GTK_ENTRY (lookup_widget (window_detail, "wineCountryEntry"))));
	if (strlen(countryItems) == 0) return;

	memset(appitems,'\0',appellation_size);
	strcpy (appitems,	gtk_entry_get_text (GTK_ENTRY (lookup_widget (window_detail, "vinapentry"))));
	memset(regitems,'\0',region_size);
	strcpy (regitems,	gtk_entry_get_text (GTK_ENTRY (lookup_widget (window_detail, "vinregentry"))));

	/* 2.2.7 */
	if (strlen(appitems) && strlen(regitems) != 0) return;

	gtk_signal_handler_block (GTK_OBJECT (lookup_widget (window_detail, "vinapentry")), signal_id_app);
	gtk_signal_handler_block (GTK_OBJECT (lookup_widget (window_detail, "vinregentry")), signal_id_reg);
	gtk_signal_handler_block (GTK_OBJECT (lookup_widget (window_detail, "wineCountryEntry")), signal_id_pay);
	

	/* need to fillup combo region for this country */
	g_list_foreach(list_country, xcave_find_C_country, countryItems);

	/* display on screen app combo */
	ptr_items = g_malloc(appellation_size);
	if (strlen(appitems) != 0) {
		strcpy(ptr_items, appitems);
		if (g_list_find_custom(tmp_list_combo_app, ptr_items, (GCompareFunc) g_ascii_strcasecmp) == NULL)
			g_free(ptr_items);
		else {
			tmp_list_combo_app = g_list_prepend (tmp_list_combo_app, ptr_items);
		}
	}
	else {
		// if (g_list_length(tmp_list_combo_app) == 0) {
			strcpy(ptr_items, "");
			tmp_list_combo_app = g_list_prepend (tmp_list_combo_app, ptr_items);
		// }
	}
	if (g_list_length(tmp_list_combo_app) != 0)
		gtk_combo_set_popdown_strings(GTK_COMBO (lookup_widget (window_detail, "vinaplist")),
									  tmp_list_combo_app);

	/* display on screen region combo */
	ptr_items = g_malloc(region_size);
	if (strlen(regitems) != 0) {
		strcpy(ptr_items, regitems);
		if (g_list_find_custom(tmp_list_combo_region, ptr_items, (GCompareFunc) g_ascii_strcasecmp) == NULL)
			g_free(ptr_items);
		else {
			tmp_list_combo_region = g_list_prepend (tmp_list_combo_region, ptr_items);
		}
	}
	else {
		//if (g_list_length(tmp_list_combo_region) == 0) {
			strcpy(ptr_items, "");
			tmp_list_combo_region = g_list_prepend (tmp_list_combo_region, ptr_items);
		// }
	}
	if (g_list_length(tmp_list_combo_region) != 0)
		gtk_combo_set_popdown_strings(GTK_COMBO (lookup_widget (window_detail, "vinreglist")),
									  tmp_list_combo_region);

	tmp_list_combo_region	= xcave_g_list_free(tmp_list_combo_region);
	tmp_list_combo_app		= xcave_g_list_free(tmp_list_combo_app);

	gtk_signal_handler_unblock (GTK_OBJECT (lookup_widget (window_detail, "vinapentry")), signal_id_app);
	gtk_signal_handler_unblock (GTK_OBJECT (lookup_widget (window_detail, "vinregentry")),signal_id_reg);
	gtk_signal_handler_unblock (GTK_OBJECT (lookup_widget (window_detail, "wineCountryEntry")),signal_id_pay);

    XCAVE_DEBUG_E;
}

/* ****************************************************************************
 * Traitement des combo vinaplist et wineCountryList suite a une modification ou une 
 * saisie de vinaplist
 */

void
xcave_find_R_country(gpointer	list_country_data, gpointer regitems)
{
	gchar *ptr_countryItems;

	if (strcasecmp(regitems, list_country_data + country_size) == 0) {
		/* need to fillup combo country for this region */
		ptr_countryItems = g_malloc(country_size);
		strncpy(ptr_countryItems, list_country_data, country_size);
		tmp_list_combo_country = g_list_append(tmp_list_combo_country,ptr_countryItems);
	}

}

void
xcave_find_R_app(gpointer	list_app_data, gpointer regitems)
{
	gchar *ptr_appitems;

	if (strcasecmp(regitems, list_app_data) == 0) {
		/* need to fillup combo country for this region */
		ptr_appitems = g_malloc(appellation_size);
		strncpy(ptr_appitems, list_app_data + region_size, appellation_size);
		tmp_list_combo_app = g_list_append(tmp_list_combo_app,ptr_appitems);
	}
}

void
xcave_find_R_reg(gpointer	list_region_data, gpointer regitems)
{
	if (strcasecmp(regitems, list_region_data) == 0) {
		/* need to fillup combo country for this region */
		g_list_foreach(list_country_region,xcave_find_R_country,regitems);

		/* need to fillup combo appellation for this region */
		g_list_foreach(list_region_appellation,xcave_find_R_app,regitems);
	}
}

/*
 * Traitement d'un signal
 * Windows: 'detail'
 * GtkEntry: vinregentry
 * Comment: Selection d'une region differente dans la GtkCombo vinreglist
 */
void
on_vinregentry_changed                 (GtkEditable     *editable,
                                        gpointer         user_data)
{
	gchar	*ptr_items;
	gchar	appitems[appellation_size];
	gchar	regitems[region_size];
	gchar	countryItems[country_size];

    XCAVE_DEBUG_B;

	/* 
	* Une region est selectionnee - 
	*   Si region pas encore selectionne - alors on selectionne la region associee a l appellation
	*   Si country pas encore selectionne - alors on selectionne le country associe a la region
	*/

	memset(regitems,'\0',region_size);
	strcpy (regitems,	gtk_entry_get_text (GTK_ENTRY (lookup_widget (window_detail, "vinregentry"))));
	if (strlen(regitems) == 0) return;
		
	memset(appitems,'\0',appellation_size);
	strcpy (appitems,	gtk_entry_get_text (GTK_ENTRY (lookup_widget (window_detail, "vinapentry"))));
	memset(countryItems,'\0',country_size);
	strcpy (countryItems,	gtk_entry_get_text (GTK_ENTRY (lookup_widget (window_detail, "wineCountryEntry"))));

	gtk_signal_handler_block (GTK_OBJECT (lookup_widget (window_detail, "vinapentry")), signal_id_app);
	gtk_signal_handler_block (GTK_OBJECT (lookup_widget (window_detail, "vinregentry")), signal_id_reg);
	gtk_signal_handler_block (GTK_OBJECT (lookup_widget (window_detail, "wineCountryEntry")), signal_id_pay);
	
	/* need to fillup app and country combos for this region */
	g_list_foreach(list_region,xcave_find_R_reg,regitems);

	/* display on screen app combo */
	ptr_items = g_malloc(appellation_size);
	if (strlen(appitems) != 0) {
		strcpy(ptr_items, appitems);
		if (g_list_find_custom(tmp_list_combo_app, ptr_items, (GCompareFunc) g_ascii_strcasecmp) == NULL) {
			g_free(ptr_items);
		}
		else { 
			tmp_list_combo_app = g_list_prepend (tmp_list_combo_app, ptr_items);
		}
	}
	else {
		//if (g_list_length(tmp_list_combo_app) == 0) {
			strcpy(ptr_items, "");
			tmp_list_combo_app = g_list_prepend (tmp_list_combo_app, ptr_items);
		//}
	}
	if (g_list_length(tmp_list_combo_app) != 0)
		gtk_combo_set_popdown_strings(GTK_COMBO (lookup_widget (window_detail, "vinaplist")),
									  tmp_list_combo_app);

	/* display on screen country combo */
	ptr_items = g_malloc(country_size);
	if (strlen(countryItems) != 0) {
		strcpy(ptr_items, countryItems); 
		if (g_list_find_custom(tmp_list_combo_country, ptr_items, (GCompareFunc) g_ascii_strcasecmp) == NULL) {
			g_free(ptr_items);
		}
	}
	else {
		if (g_list_length(tmp_list_combo_country) == 0) {
			strcpy(ptr_items, "");
			tmp_list_combo_country = g_list_prepend (tmp_list_combo_country, ptr_items);
		}
	}
	
	if (g_list_length(tmp_list_combo_country) != 0)
		gtk_combo_set_popdown_strings(GTK_COMBO (lookup_widget (window_detail, "wineCountryList")),
									  tmp_list_combo_country);

	tmp_list_combo_country	= xcave_g_list_free(tmp_list_combo_country);
	tmp_list_combo_app	= xcave_g_list_free(tmp_list_combo_app);

	gtk_signal_handler_unblock (GTK_OBJECT (lookup_widget (window_detail, "vinapentry")), signal_id_app);
	gtk_signal_handler_unblock (GTK_OBJECT (lookup_widget (window_detail, "vinregentry")), signal_id_reg);
	gtk_signal_handler_unblock (GTK_OBJECT (lookup_widget (window_detail, "wineCountryEntry")),signal_id_pay);


    XCAVE_DEBUG_E;
}

/* *****************************************************************************
 * Traitement des combo wineCountryList et vinreglist suite a une modification ou une 
 * saisie de vinaplist
 */
void
xcave_find_A_country(gpointer	list_country_data, gpointer regitems)
{
	gchar *ptr_countryItems;

	if (strcasecmp(regitems, list_country_data + region_size) == 0) {	
		/* need to fillup combo country for this region */
		ptr_countryItems = g_malloc(country_size);
		strncpy(ptr_countryItems, list_country_data, country_size);
		if (g_list_find_custom(tmp_list_combo_country, ptr_countryItems, (GCompareFunc) g_ascii_strcasecmp) == NULL)
			tmp_list_combo_country = g_list_append(tmp_list_combo_country,ptr_countryItems);
	}
}

void
xcave_find_A_region(gpointer	list_region_data, gpointer appitems)
{
	gchar *ptr_regitems;

	if (strcasecmp(appitems, list_region_data + region_size) == 0) {
		
		/* need to fillup combo country for this region */
		ptr_regitems = g_malloc(region_size);
		strncpy(ptr_regitems, list_region_data, region_size);
		tmp_list_combo_region = g_list_append(tmp_list_combo_region,ptr_regitems);

		/* need to know originated country */
		g_list_foreach(list_country_region,xcave_find_A_country,ptr_regitems);
	}

}

void
xcave_find_A_app(gpointer	list_appellation_data, gpointer appitems)
{

	if (strcasecmp(appitems, list_appellation_data) == 0) {
	
		/* need to know originated region */
		g_list_foreach(list_region_appellation,xcave_find_A_region,appitems);
	}
}

/*
 * Traitement d'un signal
 * Windows: 'detail'
 * GtkEntry: vinapentry
 * Comment: Selection ou modification d'une appellation (GtkCombo vinaplist)
 */
void
on_vinapentry_changed                  (GtkEditable     *editable,
                                        gpointer         user_data)
{

	gchar	appitems[appellation_size];
	gchar	regitems[region_size];
	gchar	countryItems[country_size];
	gchar	*ptr_items;

  
    XCAVE_DEBUG_B;
	/* 
	* Une appelation est selectionnee - 
	*   Si region pas encore selectionne - alors on selectionne la region associee a l appellation
	*   Si country pas encore selectionne - alors on selectionne le country associe a la region
	*/

	memset(appitems,'\0',appellation_size);
	strcpy (appitems,	gtk_entry_get_text (GTK_ENTRY (lookup_widget (window_detail, "vinapentry"))));
	if (strlen(appitems) == 0) return;
		
	memset(regitems,'\0',region_size);
	strcpy (regitems,	gtk_entry_get_text (GTK_ENTRY (lookup_widget (window_detail, "vinregentry"))));
	memset(countryItems,'\0',country_size);
	strcpy (countryItems,	gtk_entry_get_text (GTK_ENTRY (lookup_widget (window_detail, "wineCountryEntry"))));

	/* 2.2.7 */
	if (strlen(countryItems) && strlen(regitems) != 0) return;

	gtk_signal_handler_block (GTK_OBJECT (lookup_widget (window_detail, "wineCountryEntry")), signal_id_pay);
	gtk_signal_handler_block (GTK_OBJECT (lookup_widget (window_detail, "vinregentry")), signal_id_reg);
	gtk_signal_handler_block (GTK_OBJECT (lookup_widget (window_detail, "vinapentry")), signal_id_app);

	/* need to fillup combo region for this appellation */
	g_list_foreach(list_appellation,xcave_find_A_app,appitems);

	/* display on screen region combo */
	ptr_items = g_malloc(region_size);
	if (strlen(regitems) != 0) {
		strcpy(ptr_items, regitems);
		if (g_list_find_custom(tmp_list_combo_region, ptr_items, (GCompareFunc) g_ascii_strcasecmp) == NULL)
			g_free(ptr_items);
		else {
			tmp_list_combo_region = g_list_prepend (tmp_list_combo_region, ptr_items);
		}
	}
	else {
		if (g_list_length(tmp_list_combo_region) == 0) {
			strcpy(ptr_items, "");
			tmp_list_combo_region = g_list_prepend (tmp_list_combo_region, ptr_items);
		}
	}
	if (g_list_length(tmp_list_combo_region) != 0)
		gtk_combo_set_popdown_strings(GTK_COMBO (lookup_widget (window_detail, "vinreglist")),
									  tmp_list_combo_region);

	/* display on screen country combo */
	ptr_items = g_malloc(country_size);
	if (strlen(countryItems) != 0) {
		strcpy(ptr_items, countryItems);
		if (g_list_find_custom(tmp_list_combo_country, ptr_items, (GCompareFunc) g_ascii_strcasecmp) == NULL)
			g_free(ptr_items);
		else {
			tmp_list_combo_country = g_list_prepend (tmp_list_combo_country, ptr_items);
		}
	}
	else {
		if (g_list_length(tmp_list_combo_country) == 0) {
			strcpy(ptr_items, "");
			tmp_list_combo_country = g_list_prepend (tmp_list_combo_country, ptr_items);
		}
	}
	
	if (g_list_length(tmp_list_combo_country) != 0)
		gtk_combo_set_popdown_strings(GTK_COMBO (lookup_widget (window_detail, "wineCountryList")),
									  tmp_list_combo_country);

	tmp_list_combo_country   = xcave_g_list_free(tmp_list_combo_country);
	tmp_list_combo_region = xcave_g_list_free(tmp_list_combo_region);

	gtk_signal_handler_unblock (GTK_OBJECT (lookup_widget (window_detail, "wineCountryEntry")), signal_id_pay);
	gtk_signal_handler_unblock (GTK_OBJECT (lookup_widget (window_detail, "vinregentry")),signal_id_reg);
	gtk_signal_handler_unblock (GTK_OBJECT (lookup_widget (window_detail, "vinapentry")),signal_id_app);


    XCAVE_DEBUG_E;
}

void
on_window_supp_destroy   (GtkObject       *object,
                          gpointer         user_data)
{

   gtk_object_destroy (GTK_OBJECT (confirm));

   /* 
    * reactive window_xcave
    */
   gtk_widget_set_sensitive(window_xcave, TRUE);
}
